@@ -135,15 +135,34 @@ raop_rsa_encrypt (guchar *text, gint len, guchar *res)
 		0x5e,0xf,0xc8,0x75,0x34,0x3e,0xc7,0x82,0x11,0x76,0x25,0xcd
 		,0xbf,0x98,0x44,0x7b};
 	static const guchar exp[] = {0x01, 0x00, 0x01};
+	BIGNUM *n, *e;
 
 	rsa = RSA_new ();
-	rsa->n = BN_bin2bn (mod, 256, NULL);
-	rsa->e = BN_bin2bn (exp, 3, NULL);
+	n = BN_bin2bn (mod, 256, NULL);
+	e = BN_bin2bn (exp, 3, NULL);
+	if (!rsa || !n || !e)
+		goto err;
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000
+	rsa->n = n;
+	rsa->e = e;
+#else
+	if (!RSA_set0_key(rsa, n, e, NULL))
+		goto err;
+#endif
 
 	size = RSA_public_encrypt (len, text, res, rsa, RSA_PKCS1_OAEP_PADDING);
 
 	RSA_free (rsa);
 	return size;
+err:
+	if (rsa)
+		RSA_free(rsa);
+	if (n)
+		BN_free(n);
+	if (e)
+		BN_free(e);
+	return 0;
 }
 
 static void
@@ -250,6 +269,8 @@ raop_rtsp_announce (raop_client_t *rc)
 	gint ret = RAOP_EOK;
 
 	size = raop_rsa_encrypt (rc->aes_key_str, 16, enc_aes_key);
+	if (size == 0)
+		return RAOP_EFAIL;
 
 	size = b64_encode_alloc (enc_aes_key, size, &key);
 	g_strdelimit (key, "=", '\0');
@@ -397,10 +418,16 @@ raop_client_init (raop_client_t **client)
 
 	/* setup client_id, aes_key */
 	ret = RAND_bytes (rand_buf, sizeof (rand_buf));
+	if (ret <= 0) {
+		return RAOP_EFAIL;
+	}
 	g_snprintf (rc->client_id, 17, "%08X%08X", *((guint *) rand_buf),
 	            *((guint *) (rand_buf + 4)));
 
 	ret = RAND_bytes (rc->aes_key_str, sizeof (rc->aes_key_str));
+	if (ret <= 0) {
+		return RAOP_EFAIL;
+	}
 	rc->aes_key = (AES_KEY *) g_malloc (sizeof (AES_KEY));
 	AES_set_encrypt_key (rc->aes_key_str, 128, rc->aes_key);
 
