@@ -1,5 +1,5 @@
 /*  XMMS2 - X Music Multiplexer System
- *  Copyright (C) 2003-2011 XMMS2 Team
+ *  Copyright (C) 2003-2023 XMMS2 Team
  *
  *  PLUGINS ARE NOT CONSIDERED TO BE DERIVED WORK !!!
  *
@@ -14,11 +14,11 @@
  *  Lesser General Public License for more details.
  */
 
-#include "xmms/xmms_xformplugin.h"
-#include "xmms/xmms_sample.h"
-#include "xmms/xmms_log.h"
-#include "xmms/xmms_medialib.h"
-#include "xmms/xmms_bindata.h"
+#include <xmms/xmms_xformplugin.h>
+#include <xmms/xmms_sample.h>
+#include <xmms/xmms_log.h>
+#include <xmms/xmms_medialib.h>
+#include <xmms/xmms_bindata.h>
 
 #include <string.h>
 #include <math.h>
@@ -26,33 +26,6 @@
 
 #include <glib.h>
 
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-# define FLAC__STREAM_DECODER_SEEK_STATUS_OK FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_OK
-# define FLAC__STREAM_DECODER_SEEK_STATUS_ERROR FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_ERROR
-# define FLAC__STREAM_DECODER_TELL_STATUS_OK FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_OK
-# define FLAC__STREAM_DECODER_TELL_STATUS_ERROR FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR
-# define FLAC__STREAM_DECODER_LENGTH_STATUS_OK FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK
-# define FLAC__STREAM_DECODER_LENGTH_STATUS_ERROR FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_ERROR
-# define FLAC__STREAM_DECODER_END_OF_STREAM FLAC__SEEKABLE_STREAM_DECODER_END_OF_STREAM
-# define FLAC__StreamDecoder FLAC__SeekableStreamDecoder
-# define FLAC__StreamDecoderState FLAC__SeekableStreamDecoderState
-# define FLAC__StreamDecoderReadStatus FLAC__SeekableStreamDecoderReadStatus
-# define FLAC__StreamDecoderTellStatus FLAC__SeekableStreamDecoderTellStatus
-# define FLAC__StreamDecoderSeekStatus FLAC__SeekableStreamDecoderSeekStatus
-# define FLAC__StreamDecoderLengthStatus FLAC__SeekableStreamDecoderLengthStatus
-# define FLAC__stream_decoder_new FLAC__seekable_stream_decoder_new
-# define FLAC__stream_decoder_set_metadata_respond_all FLAC__seekable_stream_decoder_set_metadata_respond_all
-# define FLAC__stream_decoder_finish FLAC__seekable_stream_decoder_finish
-# define FLAC__stream_decoder_delete FLAC__seekable_stream_decoder_delete
-# define FLAC__stream_decoder_process_single FLAC__seekable_stream_decoder_process_single
-# define FLAC__stream_decoder_get_state FLAC__seekable_stream_decoder_get_state
-# define FLAC__stream_decoder_seek_absolute FLAC__seekable_stream_decoder_seek_absolute
-# define FLAC__stream_decoder_process_until_end_of_metadata FLAC__seekable_stream_decoder_process_until_end_of_metadata
-typedef unsigned read_callback_size_t;
-#else
-typedef size_t read_callback_size_t;
-#endif
-
 typedef struct xmms_flac_data_St {
 	FLAC__StreamDecoder *flacdecoder;
 	FLAC__StreamMetadata *vorbiscomment;
@@ -76,14 +49,63 @@ static gboolean xmms_flac_init (xmms_xform_t *xform);
 static void xmms_flac_destroy (xmms_xform_t *xform);
 static gint64 xmms_flac_seek (xmms_xform_t *xform, gint64 samples, xmms_xform_seek_mode_t whence, xmms_error_t *err);
 
+/** These are the properties that we extract from the comments */
+static const xmms_xform_metadata_basic_mapping_t mappings[] = {
+	{ "album",                     XMMS_MEDIALIB_ENTRY_PROPERTY_ALBUM             },
+	{ "title",                     XMMS_MEDIALIB_ENTRY_PROPERTY_TITLE             },
+	{ "artist",                    XMMS_MEDIALIB_ENTRY_PROPERTY_ARTIST            },
+	{ "albumartist",               XMMS_MEDIALIB_ENTRY_PROPERTY_ALBUM_ARTIST      },
+	{ "date",                      XMMS_MEDIALIB_ENTRY_PROPERTY_YEAR              },
+	{ "originaldate",              XMMS_MEDIALIB_ENTRY_PROPERTY_ORIGINALYEAR      },
+	{ "composer",                  XMMS_MEDIALIB_ENTRY_PROPERTY_COMPOSER          },
+	{ "lyricist",                  XMMS_MEDIALIB_ENTRY_PROPERTY_LYRICIST          },
+	{ "conductor",                 XMMS_MEDIALIB_ENTRY_PROPERTY_CONDUCTOR         },
+	{ "performer",                 XMMS_MEDIALIB_ENTRY_PROPERTY_PERFORMER         },
+	{ "remixer",                   XMMS_MEDIALIB_ENTRY_PROPERTY_REMIXER           },
+	{ "arranger",                  XMMS_MEDIALIB_ENTRY_PROPERTY_ARRANGER          },
+	{ "producer",                  XMMS_MEDIALIB_ENTRY_PROPERTY_PRODUCER          },
+	{ "mixer",                     XMMS_MEDIALIB_ENTRY_PROPERTY_MIXER             },
+	{ "grouping",                  XMMS_MEDIALIB_ENTRY_PROPERTY_GROUPING          },
+	{ "grouping",                  XMMS_MEDIALIB_ENTRY_PROPERTY_GROUPING          },
+	{ "tracknumber",               XMMS_MEDIALIB_ENTRY_PROPERTY_TRACKNR           },
+	{ "tracktotal",                XMMS_MEDIALIB_ENTRY_PROPERTY_TOTALTRACKS       },
+	{ "totaltracks",               XMMS_MEDIALIB_ENTRY_PROPERTY_TOTALTRACKS       },
+	{ "discnumber",                XMMS_MEDIALIB_ENTRY_PROPERTY_PARTOFSET         },
+	{ "disctotal",                 XMMS_MEDIALIB_ENTRY_PROPERTY_TOTALSET          },
+	{ "totaldiscs",                XMMS_MEDIALIB_ENTRY_PROPERTY_TOTALSET          },
+	{ "compilation",               XMMS_MEDIALIB_ENTRY_PROPERTY_COMPILATION       },
+	{ "comment",                   XMMS_MEDIALIB_ENTRY_PROPERTY_COMMENT           },
+	{ "genre",                     XMMS_MEDIALIB_ENTRY_PROPERTY_GENRE             },
+	{ "bpm",                       XMMS_MEDIALIB_ENTRY_PROPERTY_BPM               },
+	{ "isrc",                      XMMS_MEDIALIB_ENTRY_PROPERTY_ISRC              },
+	{ "copyright",                 XMMS_MEDIALIB_ENTRY_PROPERTY_COPYRIGHT         },
+	{ "catalognumber",             XMMS_MEDIALIB_ENTRY_PROPERTY_CATALOGNUMBER     },
+	{ "barcode",                   XMMS_MEDIALIB_ENTRY_PROPERTY_BARCODE           },
+	{ "albumsort",                 XMMS_MEDIALIB_ENTRY_PROPERTY_ALBUM_SORT        },
+	{ "albumartistsort",           XMMS_MEDIALIB_ENTRY_PROPERTY_ALBUM_ARTIST_SORT },
+	{ "artistsort",                XMMS_MEDIALIB_ENTRY_PROPERTY_ARTIST_SORT       },
+	{ "description",               XMMS_MEDIALIB_ENTRY_PROPERTY_DESCRIPTION       },
+	{ "label",                     XMMS_MEDIALIB_ENTRY_PROPERTY_PUBLISHER         },
+	{ "asin",                      XMMS_MEDIALIB_ENTRY_PROPERTY_ASIN              },
+	{ "conductor",                 XMMS_MEDIALIB_ENTRY_PROPERTY_CONDUCTOR         },
+	{ "musicbrainz_albumid",       XMMS_MEDIALIB_ENTRY_PROPERTY_ALBUM_ID          },
+	{ "musicbrainz_artistid",      XMMS_MEDIALIB_ENTRY_PROPERTY_ARTIST_ID         },
+	{ "musicbrainz_trackid",       XMMS_MEDIALIB_ENTRY_PROPERTY_TRACK_ID          },
+	{ "musicbrainz_albumartistid", XMMS_MEDIALIB_ENTRY_PROPERTY_COMPILATION       },
+	{ "replaygain_track_gain",     XMMS_MEDIALIB_ENTRY_PROPERTY_GAIN_TRACK        },
+	{ "replaygain_album_gain",     XMMS_MEDIALIB_ENTRY_PROPERTY_GAIN_ALBUM        },
+	{ "replaygain_track_peak",     XMMS_MEDIALIB_ENTRY_PROPERTY_PEAK_TRACK        },
+	{ "replaygain_album_peak",     XMMS_MEDIALIB_ENTRY_PROPERTY_PEAK_ALBUM        },
+};
+
 /*
  * Plugin header
  */
 
-XMMS_XFORM_PLUGIN ("flac",
-                   "FLAC Decoder", XMMS_VERSION,
-                   "Free Lossless Audio Codec decoder",
-                   xmms_flac_plugin_setup);
+XMMS_XFORM_PLUGIN_DEFINE ("flac",
+                          "FLAC Decoder", XMMS_VERSION,
+                          "Free Lossless Audio Codec decoder",
+                          xmms_flac_plugin_setup);
 
 static gboolean
 xmms_flac_plugin_setup (xmms_xform_plugin_t *xform_plugin)
@@ -99,10 +121,13 @@ xmms_flac_plugin_setup (xmms_xform_plugin_t *xform_plugin)
 
 	xmms_xform_plugin_methods_set (xform_plugin, &methods);
 
+	xmms_xform_plugin_metadata_basic_mapper_init (xform_plugin, mappings,
+	                                              G_N_ELEMENTS (mappings));
+
 	xmms_xform_plugin_indata_add (xform_plugin,
 	                              XMMS_STREAM_TYPE_MIMETYPE,
 	                              "audio/x-flac",
-	                              NULL);
+	                              XMMS_STREAM_TYPE_END);
 
 	xmms_magic_add ("flac header", "audio/x-flac",
 	                "0 string fLaC", NULL);
@@ -111,33 +136,26 @@ xmms_flac_plugin_setup (xmms_xform_plugin_t *xform_plugin)
 }
 
 static FLAC__StreamDecoderReadStatus
-flac_callback_read (const FLAC__StreamDecoder *flacdecoder,
-                    FLAC__byte buffer[],
-                    read_callback_size_t *bytes,
-                    void *client_data)
+flac_callback_read (const FLAC__StreamDecoder *flacdecoder, FLAC__byte buffer[],
+                    size_t *bytes, void *client_data)
 {
 	xmms_xform_t *xform = (xmms_xform_t *) client_data;
 	xmms_error_t error;
 	gint ret;
 
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-	g_return_val_if_fail (xform,
-	                      FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_ERROR);
-#else
-	g_return_val_if_fail (xform,
-	                      FLAC__STREAM_DECODER_READ_STATUS_ABORT);
-#endif
+	g_return_val_if_fail (xform, FLAC__STREAM_DECODER_READ_STATUS_ABORT);
 
 	ret = xmms_xform_read (xform, (gchar *)buffer, *bytes, &error);
-	*bytes = ret;
-
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-	return (ret <= 0) ? FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_ERROR
-	                  : FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_OK;
-#else
-	return (ret <= 0) ? FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM
-	                  : FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
-#endif
+	if (ret == 0) {
+		*bytes = 0;
+		return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
+	} else if (ret < 0) {
+		*bytes = 0;
+		return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
+	} else {
+		*bytes = ret;
+		return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
+	}
 }
 
 static FLAC__StreamDecoderWriteStatus
@@ -275,7 +293,6 @@ flac_callback_metadata (const FLAC__StreamDecoder *flacdecoder,
 		case FLAC__METADATA_TYPE_VORBIS_COMMENT:
 			data->vorbiscomment = FLAC__metadata_object_clone (metadata);
 			break;
-#if defined(FLAC_API_VERSION_CURRENT) && FLAC_API_VERSION_CURRENT > 7
 		case FLAC__METADATA_TYPE_PICTURE: {
 			gchar hash[33];
 			if (metadata->data.picture.type == FLAC__STREAM_METADATA_PICTURE_TYPE_FRONT_COVER &&
@@ -283,16 +300,15 @@ flac_callback_metadata (const FLAC__StreamDecoder *flacdecoder,
 			                             metadata->data.picture.data_length,
 			                             hash)) {
 				const gchar *metakey;
-				
+
 				metakey = XMMS_MEDIALIB_ENTRY_PROPERTY_PICTURE_FRONT;
 				xmms_xform_metadata_set_str (xform, metakey, hash);
-				
+
 				metakey = XMMS_MEDIALIB_ENTRY_PROPERTY_PICTURE_FRONT_MIME;
 				xmms_xform_metadata_set_str (xform, metakey, metadata->data.picture.mime_type);
 			}
 			break;
 		}
-#endif
 		/* if we want to support more metadata types here,
 		 * don't forget to add a call to
 		 * FLAC__stream_decoder_set_metadata_respond() below.
@@ -327,71 +343,7 @@ flac_callback_error (const FLAC__StreamDecoder *flacdecoder,
 	XMMS_DBG ("%s", FLAC__StreamDecoderErrorStatusString[status]);
 }
 
-typedef enum { STRING, INTEGER, RPGAIN } ptype;
-typedef struct {
-	const gchar *vname;
-	const gchar *xname;
-	ptype type;
-} props;
 
-#define MUSICBRAINZ_VA_ID "89ad4ac3-39f7-470e-963a-56509c546377"
-
-/** These are the properties that we extract from the comments */
-static const props properties[] = {
-	{ "title",                XMMS_MEDIALIB_ENTRY_PROPERTY_TITLE,     STRING  },
-	{ "artist",               XMMS_MEDIALIB_ENTRY_PROPERTY_ARTIST,    STRING  },
-	{ "album",                XMMS_MEDIALIB_ENTRY_PROPERTY_ALBUM,     STRING  },
-	{ "tracknumber",          XMMS_MEDIALIB_ENTRY_PROPERTY_TRACKNR,   INTEGER },
-	{ "date",                 XMMS_MEDIALIB_ENTRY_PROPERTY_YEAR,      STRING  },
-	{ "genre",                XMMS_MEDIALIB_ENTRY_PROPERTY_GENRE,     STRING  },
-	{ "comment",              XMMS_MEDIALIB_ENTRY_PROPERTY_COMMENT,   STRING  },
-	{ "description",          XMMS_MEDIALIB_ENTRY_PROPERTY_DESCRIPTION, STRING  },
-	{ "discnumber",           XMMS_MEDIALIB_ENTRY_PROPERTY_PARTOFSET, INTEGER },
-	{ "musicbrainz_albumid",  XMMS_MEDIALIB_ENTRY_PROPERTY_ALBUM_ID,  STRING  },
-	{ "musicbrainz_artistid", XMMS_MEDIALIB_ENTRY_PROPERTY_ARTIST_ID, STRING  },
-	{ "musicbrainz_trackid",  XMMS_MEDIALIB_ENTRY_PROPERTY_TRACK_ID,  STRING  },
-	{ "replaygain_track_gain",XMMS_MEDIALIB_ENTRY_PROPERTY_GAIN_TRACK,RPGAIN  },
-	{ "replaygain_album_gain",XMMS_MEDIALIB_ENTRY_PROPERTY_GAIN_ALBUM,RPGAIN  },
-	{ "replaygain_track_peak",XMMS_MEDIALIB_ENTRY_PROPERTY_PEAK_TRACK,STRING  },
-	{ "replaygain_album_peak",XMMS_MEDIALIB_ENTRY_PROPERTY_PEAK_ALBUM,STRING  },
-};
-
-/* note that "key" is NOT NUL-terminated here,
- * but "value" is.
- */
-static void
-handle_comment (xmms_xform_t *xform,
-                const gchar *key, gint key_len, const gchar *value)
-{
-	gchar buf[8];
-	gint i;
-
-	for (i = 0; i < G_N_ELEMENTS (properties); i++) {
-		if (key_len != strlen (properties[i].vname))
-			continue;
-		if ((!g_ascii_strncasecmp (key, "MUSICBRAINZ_ALBUMARTISTID", key_len)) &&
-		    (!g_ascii_strcasecmp (value, MUSICBRAINZ_VA_ID))) {
-			const gchar *metakey = XMMS_MEDIALIB_ENTRY_PROPERTY_COMPILATION;
-			xmms_xform_metadata_set_int (xform, metakey, 1);
-		} else if (!g_ascii_strncasecmp (key, properties[i].vname, key_len)) {
-			if (properties[i].type == INTEGER) {
-				gint tmp = strtol (value, NULL, 10);
-				xmms_xform_metadata_set_int (xform,
-				                             properties[i].xname, tmp);
-			} else if (properties[i].type == RPGAIN) {
-				g_snprintf (buf, sizeof (buf), "%f",
-				            pow (10.0, g_strtod (value, NULL) / 20));
-
-				/** @todo this should probably be a int instead? */
-				xmms_xform_metadata_set_str (xform,
-				                             properties[i].xname, buf);
-			} else {
-				xmms_xform_metadata_set_str (xform,
-				                             properties[i].xname, value);
-			}
-		}
-	}
-}
 
 static void
 handle_comments (xmms_xform_t *xform, xmms_flac_data_t *data)
@@ -404,23 +356,27 @@ handle_comments (xmms_xform_t *xform, xmms_flac_data_t *data)
 	vc = &data->vorbiscomment->data.vorbis_comment;
 
 	for (i = 0; i < vc->num_comments; i++) {
-		FLAC__byte *ptr, *content = vc->comments[i].entry;
-		gint j;
+		const gchar *entry, *ptr;
+		gchar key[64];
+		gsize length;
 
-		/* check whether it's a valid comment */
-		if (!content || !*content || *content == '=')
+		entry = (const gchar *) vc->comments[i].entry;
+		length = vc->comments[i].length;
+
+		if (entry == NULL || *entry == '\0')
 			continue;
 
-		for (ptr = content, j = 0; j < vc->comments[i].length; ptr++, j++)
-			if (*ptr == '=')
-				break;
-
-		if (j == vc->comments[i].length)
+		ptr = memchr (entry, '=', length);
+		if (ptr == NULL)
 			continue;
 
-		handle_comment (xform,
-		                (gchar *) content, j,
-		                (gchar *) ptr + 1);
+		ptr++;
+
+		g_strlcpy (key, entry, MIN (ptr - entry, sizeof (key)));
+
+		if (!xmms_xform_metadata_mapper_match (xform, key, ptr, length - (ptr - entry))) {
+			XMMS_DBG ("Unhandled tag '%s'", entry);
+		}
 	}
 }
 
@@ -430,11 +386,7 @@ xmms_flac_init (xmms_xform_t *xform)
 	xmms_flac_data_t *data;
 	xmms_sample_format_t sample_fmt;
 	FLAC__bool retval;
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-	FLAC__StreamDecoderState init_status;
-#else
 	FLAC__StreamDecoderInitStatus init_status;
-#endif
 	gint filesize;
 	const gchar *metakey;
 
@@ -449,59 +401,27 @@ xmms_flac_init (xmms_xform_t *xform)
 	/* we don't need to explicitly tell the decoder to respond to
 	 * FLAC__METADATA_TYPE_STREAMINFO here, it always does.
 	 */
-#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
-	FLAC__seekable_stream_decoder_set_metadata_respond (data->flacdecoder,
-	                                                    FLAC__METADATA_TYPE_VORBIS_COMMENT);
-	FLAC__seekable_stream_decoder_set_eof_callback (data->flacdecoder,
-	                                                flac_callback_eof);
-	FLAC__seekable_stream_decoder_set_read_callback (data->flacdecoder,
-	                                                 flac_callback_read);
-	FLAC__seekable_stream_decoder_set_seek_callback (data->flacdecoder,
-	                                                 flac_callback_seek);
-	FLAC__seekable_stream_decoder_set_tell_callback (data->flacdecoder,
-	                                                 flac_callback_tell);
-	FLAC__seekable_stream_decoder_set_write_callback (data->flacdecoder,
-	                                                  flac_callback_write);
-	FLAC__seekable_stream_decoder_set_error_callback (data->flacdecoder,
-	                                                  flac_callback_error);
-	FLAC__seekable_stream_decoder_set_length_callback (data->flacdecoder,
-	                                                   flac_callback_length);
-	FLAC__seekable_stream_decoder_set_metadata_callback (data->flacdecoder,
-	                                                     flac_callback_metadata);
-
-	FLAC__seekable_stream_decoder_set_client_data (data->flacdecoder, xform);
-
-	init_status = FLAC__seekable_stream_decoder_init (data->flacdecoder);
-
-	if (init_status != FLAC__SEEKABLE_STREAM_DECODER_OK) {
-		const gchar *errmsg = FLAC__seekable_stream_decoder_get_resolved_state_string (data->flacdecoder);
-		XMMS_DBG ("FLAC init failed: %s", errmsg);
-		goto err;
-	}
-#else
 	FLAC__stream_decoder_set_metadata_respond (data->flacdecoder,
 	                                           FLAC__METADATA_TYPE_VORBIS_COMMENT);
 	FLAC__stream_decoder_set_metadata_respond (data->flacdecoder,
 	                                           FLAC__METADATA_TYPE_PICTURE);
 
-	init_status =
-		FLAC__stream_decoder_init_stream (data->flacdecoder,
-		                                  flac_callback_read,
-		                                  flac_callback_seek,
-		                                  flac_callback_tell,
-		                                  flac_callback_length,
-		                                  flac_callback_eof,
-		                                  flac_callback_write,
-		                                  flac_callback_metadata,
-		                                  flac_callback_error,
-		                                  xform);
+	init_status = FLAC__stream_decoder_init_stream (data->flacdecoder,
+	                                                flac_callback_read,
+	                                                flac_callback_seek,
+	                                                flac_callback_tell,
+	                                                flac_callback_length,
+	                                                flac_callback_eof,
+	                                                flac_callback_write,
+	                                                flac_callback_metadata,
+	                                                flac_callback_error,
+	                                                xform);
 
 	if (init_status != FLAC__STREAM_DECODER_INIT_STATUS_OK) {
 		XMMS_DBG ("FLAC init failed: %s",
 		          FLAC__stream_decoder_get_resolved_state_string (data->flacdecoder));
 		goto err;
 	}
-#endif
 
 	retval = FLAC__stream_decoder_process_until_end_of_metadata (data->flacdecoder);
 	if (!retval)
@@ -566,7 +486,6 @@ xmms_flac_read (xmms_xform_t *xform, xmms_sample_t *buf, gint len,
 {
 	FLAC__StreamDecoderState state;
 	xmms_flac_data_t *data;
-	gboolean ret;
 	guint32 size;
 
 	g_return_val_if_fail (xform, FALSE);
@@ -577,7 +496,14 @@ xmms_flac_read (xmms_xform_t *xform, xmms_sample_t *buf, gint len,
 	size = MIN (data->buffer->len, len);
 
 	if (size <= 0) {
-		ret = FLAC__stream_decoder_process_single (data->flacdecoder);
+		if (!FLAC__stream_decoder_process_single (data->flacdecoder)) {
+			switch (FLAC__stream_decoder_get_state (data->flacdecoder)) {
+				case FLAC__STREAM_DECODER_END_OF_STREAM:
+					return 0;
+				default:
+					return -1;
+			}
+		}
 	}
 
 	state = FLAC__stream_decoder_get_state (data->flacdecoder);
