@@ -1,8 +1,8 @@
-from waflib import Task, Errors, Utils
+from waflib import Task, Errors
 import os
 import gzip
 
-from TaskGen import feature,before_method
+from waflib.TaskGen import feature,before_method
 
 def gzip_func(task):
     infile = task.inputs[0].abspath()
@@ -14,7 +14,12 @@ def gzip_func(task):
     try:
         input = open(infile, 'rb')
         outf = open(outfile, 'wb')
-        output = gzip.GzipFile(os.path.basename(infile), fileobj=outf)
+        # Allow building deterministic manpages:
+        #   https://reproducible-builds.org/docs/source-date-epoch/
+        mtime = os.environ.get('SOURCE_DATE_EPOCH')
+        if mtime:
+            mtime = int(mtime)
+        output = gzip.GzipFile(os.path.basename(infile), fileobj=outf, mode='wb', mtime=mtime)
         output.write(input.read())
     finally:
         if input:
