@@ -1,4 +1,3 @@
-import Utils
 import os
 
 def add_install_flag(bld, obj):
@@ -6,5 +5,5 @@ def add_install_flag(bld, obj):
 
     if env['explicit_install_name']:
         libname = obj.env["cshlib_PATTERN"] % obj.target
-        insname = os.path.join(obj.env["PREFIX"], 'lib', libname)
+        insname = os.path.join(obj.env["LIBDIR"], libname)
         obj.env.append_unique("LINKFLAGS", ["-install_name", insname])
